@@ -63,9 +63,10 @@ namespace llvm {
     class Constant;
     class Value;
 }
-#endif
 
-void initPrecedence();
+// in expressionsem.d
+Expression *semantic(Expression *e, Scope *sc);
+#endif
 
 Expression *resolveProperties(Scope *sc, Expression *e);
 Expression *resolvePropertiesOnly(Scope *sc, Expression *e1);
@@ -146,8 +147,6 @@ public:
     static void _init();
     Expression *copy();
     virtual Expression *syntaxCopy();
-    virtual Expression *semantic(Scope *sc);
-    Expression *trySemantic(Scope *sc);
 
     // kludge for template.isExpression()
     int dyncast() const { return DYNCAST_EXPRESSION; }
@@ -226,6 +225,11 @@ public:
         return ::op_overload(this, sc);
     }
 
+    virtual bool hasCode()
+    {
+        return true;
+    }
+
     virtual void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -234,8 +238,8 @@ class IntegerExp : public Expression
 public:
     dinteger_t value;
 
+    static IntegerExp *create(Loc loc, dinteger_t value, Type *type);
     bool equals(RootObject *o);
-    Expression *semantic(Scope *sc);
     dinteger_t toInteger();
     real_t toReal();
     real_t toImaginary();
@@ -245,8 +249,6 @@ public:
     void accept(Visitor *v) { v->visit(this); }
     dinteger_t getInteger() { return value; }
     void setInteger(dinteger_t value);
-
-private:
     void normalize();
 };
 
@@ -264,8 +266,8 @@ class RealExp : public Expression
 public:
     real_t value;
 
+    static RealExp *create(Loc loc, real_t value, Type *type);
     bool equals(RootObject *o);
-    Expression *semantic(Scope *sc);
     dinteger_t toInteger();
     uinteger_t toUInteger();
     real_t toReal();
@@ -280,8 +282,8 @@ class ComplexExp : public Expression
 public:
     complex_t value;
 
+    static ComplexExp *create(Loc loc, complex_t value, Type *type);
     bool equals(RootObject *o);
-    Expression *semantic(Scope *sc);
     dinteger_t toInteger();
     uinteger_t toUInteger();
     real_t toReal();
@@ -297,7 +299,6 @@ public:
     Identifier *ident;
 
     static IdentifierExp *create(Loc loc, Identifier *ident);
-    Expression *semantic(Scope *sc);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     void accept(Visitor *v) { v->visit(this); }
@@ -315,8 +316,6 @@ public:
     Dsymbol *s;
     bool hasOverloads;
 
-    Expression *semantic(Scope *sc);
-    static Expression *resolve(Loc loc, Scope *sc, Dsymbol *s, bool hasOverloads);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     void accept(Visitor *v) { v->visit(this); }
@@ -327,7 +326,6 @@ class ThisExp : public Expression
 public:
     VarDeclaration *var;
 
-    Expression *semantic(Scope *sc);
     bool isBool(bool result);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
@@ -338,7 +336,6 @@ public:
 class SuperExp : public ThisExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -349,7 +346,6 @@ public:
     unsigned char committed;    // !=0 if type is committed
 
     bool equals(RootObject *o);
-    Expression *semantic(Scope *sc);
     bool isBool(bool result);
     StringExp *toStringExp();
     void accept(Visitor *v) { v->visit(this); }
@@ -366,8 +362,8 @@ public:
     OwnedBy ownedByCtfe;
 
     static StringExp *create(Loc loc, char *s);
+    static StringExp *create(Loc loc, void *s, size_t len);
     bool equals(RootObject *o);
-    Expression *semantic(Scope *sc);
     StringExp *toStringExp();
     StringExp *toUTF8(Scope *sc);
     int compare(RootObject *obj);
@@ -409,7 +405,6 @@ public:
 
     Expression *syntaxCopy();
     bool equals(RootObject *o);
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -421,11 +416,11 @@ public:
     Expressions *elements;
     OwnedBy ownedByCtfe;
 
+    static ArrayLiteralExp *create(Loc loc, Expressions *elements);
     Expression *syntaxCopy();
     bool equals(RootObject *o);
     Expression *getElement(d_size_t i);
     static Expressions* copyElements(Expression *e1, Expression *e2 = NULL);
-    Expression *semantic(Scope *sc);
     bool isBool(bool result);
     StringExp *toStringExp();
 
@@ -441,7 +436,6 @@ public:
 
     bool equals(RootObject *o);
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     bool isBool(bool result);
 
     void accept(Visitor *v) { v->visit(this); }
@@ -501,7 +495,6 @@ public:
     static StructLiteralExp *create(Loc loc, StructDeclaration *sd, void *elements, Type *stype = NULL);
     bool equals(RootObject *o);
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     Expression *getField(Type *type, unsigned offset);
     int getFieldIndex(Type *type, unsigned offset);
     Expression *addDtorHook(Scope *sc);
@@ -516,7 +509,6 @@ class TypeExp : public Expression
 {
 public:
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     bool checkType();
     bool checkValue();
     void accept(Visitor *v) { v->visit(this); }
@@ -528,7 +520,6 @@ public:
     ScopeDsymbol *sds;
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     bool checkType();
     bool checkValue();
     void accept(Visitor *v) { v->visit(this); }
@@ -563,8 +554,8 @@ public:
     NewDeclaration *allocator;  // allocator function
     int onstack;                // allocate on stack
 
+    static NewExp *create(Loc loc, Expression *thisexp, Expressions *newargs, Type *newtype, Expressions *arguments);
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -580,7 +571,6 @@ public:
     Expressions *arguments;     // Array of Expression's to call class constructor
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -600,7 +590,6 @@ class SymOffExp : public SymbolExp
 public:
     dinteger_t offset;
 
-    Expression *semantic(Scope *sc);
     bool isBool(bool result);
 
     void accept(Visitor *v) { v->visit(this); }
@@ -613,7 +602,6 @@ class VarExp : public SymbolExp
 public:
     static VarExp *create(Loc loc, Declaration *var, bool hasOverloads = true);
     bool equals(RootObject *o);
-    Expression *semantic(Scope *sc);
     int checkModifiable(Scope *sc, int flag);
     bool checkReadModifyWrite();
     bool isLvalue();
@@ -647,8 +635,6 @@ public:
     bool equals(RootObject *o);
     void genIdent(Scope *sc);
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
-    Expression *semantic(Scope *sc, Expressions *arguments);
     MATCH matchType(Type *to, Scope *sc, FuncExp **pfe, int flag = 0);
     const char *toChars();
     bool checkType();
@@ -668,7 +654,8 @@ public:
     Dsymbol *declaration;
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
+
+    bool hasCode();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -679,7 +666,6 @@ public:
     RootObject *obj;
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -690,14 +676,12 @@ public:
     Objects *args;
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
 class HaltExp : public Expression
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -716,7 +700,6 @@ public:
     TemplateParameters *parameters;
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -729,8 +712,6 @@ public:
     Type *att1; // Save alias this type to detect recursion
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc) = 0;
-    Expression *unaSemantic(Scope *sc);
     Expression *incompatibleTypes();
     Expression *resolveLoc(Loc loc, Scope *sc);
 
@@ -750,9 +731,6 @@ public:
     Type *att2; // Save alias this type to detect recursion
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc) = 0;
-    Expression *binSemantic(Scope *sc);
-    Expression *binSemanticProp(Scope *sc);
     Expression *incompatibleTypes();
     Expression *checkOpAssignTypes(Scope *sc);
     bool checkIntegralBin();
@@ -766,7 +744,6 @@ public:
 class BinAssignExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *ex);
@@ -779,14 +756,12 @@ public:
 class CompileExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
 class ImportExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -796,7 +771,6 @@ public:
     Expression *msg;
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -809,9 +783,6 @@ public:
     bool wantsym;       // do not replace Symbol with its initializer during semantic()
 
     static DotIdExp *create(Loc loc, Expression *e, Identifier *ident);
-    Expression *semantic(Scope *sc);
-    Expression *semanticX(Scope *sc);
-    Expression *semanticY(Scope *sc, int flag);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -820,7 +791,6 @@ class DotTemplateExp : public UnaExp
 public:
     TemplateDeclaration *td;
 
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -830,7 +800,6 @@ public:
     Declaration *var;
     bool hasOverloads;
 
-    Expression *semantic(Scope *sc);
     int checkModifiable(Scope *sc, int flag);
     bool checkReadModifyWrite();
     bool isLvalue();
@@ -846,8 +815,6 @@ public:
 
     Expression *syntaxCopy();
     bool findTempDecl(Scope *sc);
-    Expression *semantic(Scope *sc);
-    Expression *semanticY(Scope *sc, int flag);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -857,7 +824,6 @@ public:
     FuncDeclaration *func;
     bool hasOverloads;
 
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -867,7 +833,6 @@ class DotTypeExp : public UnaExp
 public:
     Dsymbol *sym;               // symbol that represents a type
 
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -883,7 +848,6 @@ public:
     static CallExp *create(Loc loc, Expression *e, Expression *earg1);
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *addDtorHook(Scope *sc);
@@ -894,7 +858,6 @@ public:
 class AddrExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -902,7 +865,6 @@ public:
 class PtrExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
     int checkModifiable(Scope *sc, int flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
@@ -914,7 +876,6 @@ public:
 class NegExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -922,7 +883,6 @@ public:
 class UAddExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -930,7 +890,6 @@ public:
 class ComExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -938,15 +897,13 @@ public:
 class NotExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
 class DeleteExp : public UnaExp
 {
-    bool isRAII;
 public:
-    Expression *semantic(Scope *sc);
+    bool isRAII;
     Expression *toBoolean(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -959,7 +916,6 @@ public:
     unsigned char mod;          // MODxxxxx
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -970,8 +926,8 @@ public:
     TypeVector *to;             // the target vector type before semantic()
     unsigned dim;               // number of elements in the vector
 
+    static VectorExp *create(Loc loc, Expression *e, Type *t);
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -983,9 +939,9 @@ public:
     VarDeclaration *lengthVar;
     bool upperIsInBounds;       // true if upr <= e1.length
     bool lowerIsLessThanUpper;  // true if lwr <= upr
+    bool arrayop;               // an array operation, rather than a slice
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     int checkModifiable(Scope *sc, int flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
@@ -998,7 +954,6 @@ public:
 class ArrayLengthExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     static Expression *rewriteOpAssign(BinExp *exp);
     void accept(Visitor *v) { v->visit(this); }
@@ -1011,14 +966,12 @@ public:
     Expression *upr;
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
 class DelegatePtrExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
@@ -1028,7 +981,6 @@ public:
 class DelegateFuncptrExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
@@ -1045,7 +997,6 @@ public:
     VarDeclaration *lengthVar;
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
 
@@ -1057,7 +1008,6 @@ public:
 class DotExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -1066,12 +1016,12 @@ class CommaExp : public BinExp
 public:
     bool isGenerated;
     bool allowCommaExp;
-    Expression *semantic(Scope *sc);
     int checkModifiable(Scope *sc, int flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
     bool isBool(bool result);
+    Expression *toBoolean(Scope *sc);
     Expression *addDtorHook(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1084,7 +1034,6 @@ public:
     bool indexIsInBounds;       // true if 0 <= e2 && e2 <= e1.length - 1
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     int checkModifiable(Scope *sc, int flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
@@ -1100,7 +1049,6 @@ public:
 class PostExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -1109,7 +1057,6 @@ public:
 class PreExp : public UnaExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -1124,7 +1071,6 @@ class AssignExp : public BinExp
 public:
     int memset;         // combination of MemorySet flags
 
-    Expression *semantic(Scope *sc);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *ex);
     Expression *toBoolean(Scope *sc);
@@ -1195,7 +1141,6 @@ public:
 class PowAssignExp : public BinAssignExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -1220,14 +1165,12 @@ public:
 class CatAssignExp : public BinAssignExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
 class AddExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1235,7 +1178,6 @@ public:
 class MinExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1243,7 +1185,6 @@ public:
 class CatExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1251,7 +1192,6 @@ public:
 class MulExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1259,7 +1199,6 @@ public:
 class DivExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1267,7 +1206,6 @@ public:
 class ModExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1275,7 +1213,6 @@ public:
 class PowExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1283,7 +1220,6 @@ public:
 class ShlExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1291,7 +1227,6 @@ public:
 class ShrExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1299,7 +1234,6 @@ public:
 class UshrExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1307,7 +1241,6 @@ public:
 class AndExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1315,7 +1248,6 @@ public:
 class OrExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1323,7 +1255,6 @@ public:
 class XorExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1331,7 +1262,6 @@ public:
 class OrOrExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
     Expression *toBoolean(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1339,7 +1269,6 @@ public:
 class AndAndExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
     Expression *toBoolean(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1347,7 +1276,6 @@ public:
 class CmpExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1355,7 +1283,6 @@ public:
 class InExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1363,7 +1290,6 @@ public:
 class RemoveExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -1372,7 +1298,6 @@ public:
 class EqualExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1382,7 +1307,6 @@ public:
 class IdentityExp : public BinExp
 {
 public:
-    Expression *semantic(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -1394,7 +1318,6 @@ public:
     Expression *econd;
 
     Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
     int checkModifiable(Scope *sc, int flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
@@ -1418,7 +1341,6 @@ public:
 class FileInitExp : public DefaultInitExp
 {
 public:
-    Expression *semantic(Scope *sc);
     Expression *resolveLoc(Loc loc, Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1426,7 +1348,6 @@ public:
 class LineInitExp : public DefaultInitExp
 {
 public:
-    Expression *semantic(Scope *sc);
     Expression *resolveLoc(Loc loc, Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1434,7 +1355,6 @@ public:
 class ModuleInitExp : public DefaultInitExp
 {
 public:
-    Expression *semantic(Scope *sc);
     Expression *resolveLoc(Loc loc, Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1442,7 +1362,6 @@ public:
 class FuncInitExp : public DefaultInitExp
 {
 public:
-    Expression *semantic(Scope *sc);
     Expression *resolveLoc(Loc loc, Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1450,7 +1369,6 @@ public:
 class PrettyFuncInitExp : public DefaultInitExp
 {
 public:
-    Expression *semantic(Scope *sc);
     Expression *resolveLoc(Loc loc, Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
