@@ -40,6 +40,12 @@ public:
     static Identifier *generateId(const char *prefix);
     static Identifier *generateId(const char *prefix, size_t i);
     static Identifier *idPool(const char *s, size_t len);
+
+    static inline Identifier *idPool(const char *s)
+    {
+        return idPool(s, strlen(s));
+    }
+
     static bool isValidIdentifier(const char *p);
     static Identifier *lookup(const char *s, size_t len);
     static void initTable();
