@@ -48,6 +48,7 @@ class TryFinallyStatement;
 class CaseStatement;
 class DefaultStatement;
 class LabelStatement;
+class StaticForeach;
 
 #if IN_LLVM
 namespace llvm
@@ -101,7 +102,6 @@ public:
     virtual bool hasBreak();
     virtual bool hasContinue();
     bool usesEH();
-    int blockExit(FuncDeclaration *func, bool mustNotThrow);
     bool comeFrom();
     bool hasCode();
     virtual Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
@@ -122,6 +122,7 @@ public:
     virtual GotoCaseStatement *isGotoCaseStatement() { return NULL; }
     virtual BreakStatement *isBreakStatement() { return NULL; }
     virtual DtorExpStatement *isDtorExpStatement() { return NULL; }
+    virtual ForwardingStatement *isForwardingStatement() { return NULL; }
     virtual void accept(Visitor *v) { v->visit(this); }
 
 #if IN_LLVM
@@ -244,6 +245,22 @@ public:
     void accept(Visitor *v) { v->visit(this); }
 };
 
+class ForwardingStatement : public Statement
+{
+    ForwardingScopeDsymbol *sym;
+    Statement *statement;
+
+    Statement *syntaxCopy();
+    Statement *getRelatedLabeled();
+    bool hasBreak();
+    bool hasContinue();
+    Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexception, Statement **sfinally);
+    Statement *last();
+    Statements *flatten(Scope *sc);
+    ForwardingStatement *isForwardingStatement() { return this; }
+    void accept(Visitor *v) { v->visit(this); }
+};
+
 class WhileStatement : public Statement
 {
 public:
@@ -368,6 +385,17 @@ public:
     void accept(Visitor *v) { v->visit(this); }
 };
 
+class StaticForeachStatement : public Statement
+{
+public:
+    StaticForeach *sfe;
+
+    Statement *syntaxCopy();
+    Statements *flatten(Scope *sc);
+
+    void accept(Visitor *v) { v->visit(this); }
+};
+
 class PragmaStatement : public Statement
 {
 public:
