@@ -167,6 +167,7 @@ Where:\n\
   -dip25           implement http://wiki.dlang.org/DIP25 (experimental)\n\
   -dip1000         implement http://wiki.dlang.org/DIP1000 (experimental)\n\
   -g               add symbolic debug info\n\
+  -gf              emit debug info for all referenced types\n\
   -gs              always emit stack frame\n"
 #if 0
 "  -gx              add stack stomp code\n"
@@ -341,6 +342,7 @@ void translateArgs(size_t originalArgc, char **originalArgv,
 
   bool vdmd = false;
   bool noFiles = true;
+  bool pic = false; // -fPIC already encountered?
 
   for (size_t i = 1; i < args.size(); i++) {
     char *p = args[i];
@@ -380,7 +382,10 @@ void translateArgs(size_t originalArgc, char **originalArgv,
       else if (strcmp(p + 1, "dylib") == 0) {
         ldcArgs.push_back("-shared");
       } else if (strcmp(p + 1, "fPIC") == 0) {
-        ldcArgs.push_back("-relocation-model=pic");
+        if (!pic) {
+          ldcArgs.push_back("-relocation-model=pic");
+          pic = true;
+        }
       } else if (strcmp(p + 1, "map") == 0) {
         goto Lnot_in_ldc;
       } else if (strcmp(p + 1, "multiobj") == 0) {
@@ -389,7 +394,9 @@ void translateArgs(size_t originalArgc, char **originalArgv,
       /* -g
        * -gc
        */
-      else if (strcmp(p + 1, "gs") == 0) {
+      else if (strcmp(p + 1, "gf") == 0) {
+        ldcArgs.push_back("-g");
+      } else if (strcmp(p + 1, "gs") == 0) {
         ldcArgs.push_back("-disable-fp-elim");
       } else if (strcmp(p + 1, "gx") == 0) {
         goto Lnot_in_ldc;
@@ -432,6 +439,8 @@ void translateArgs(size_t originalArgc, char **originalArgv,
           // ignore
         } else if (strcmp(p + 6, "avx") == 0) {
           ldcArgs.push_back("-mattr=+avx");
+        } else if (strcmp(p + 6, "avx2") == 0) {
+          ldcArgs.push_back("-mattr=+avx2");
         } else if (strcmp(p + 6, "native") == 0) {
           ldcArgs.push_back(p);
         } else {
@@ -564,6 +573,9 @@ void translateArgs(size_t originalArgc, char **originalArgv,
         ldcArgs.insert(ldcArgs.end(), args.begin() + i, args.end());
         noFiles = (i == args.size() - 1);
         break;
+      } else if (p[1] == '\0') {
+        ldcArgs.push_back("-");
+        noFiles = false;
       } else if (p[1] == 'C') {
         ldcArgs.push_back(concat("-", p + 2));
       } else {
