@@ -1377,12 +1377,17 @@ void DtoIndexBoundsCheck(Loc &loc, DValue *arr, DValue *index) {
 }
 
 void DtoBoundsCheckFailCall(IRState *irs, Loc &loc) {
-  llvm::Function *errorfn =
-      getRuntimeFunction(loc, irs->module, "_d_arraybounds");
-  irs->CreateCallOrInvoke(
-      errorfn, DtoModuleFileName(irs->func()->decl->getModule(), loc),
-      DtoConstUint(loc.linnum));
+  Module *const module = irs->func()->decl->getModule();
 
-  // the function does not return
-  irs->ir->CreateUnreachable();
+  if (global.params.betterC) {
+    DtoCAssert(module, loc, DtoConstCString("array overflow"));
+  } else {
+    llvm::Function *errorfn =
+        getRuntimeFunction(loc, irs->module, "_d_arraybounds");
+    irs->CreateCallOrInvoke(errorfn, DtoModuleFileName(module, loc),
+                            DtoConstUint(loc.linnum));
+
+    // the function does not return
+    irs->ir->CreateUnreachable();
+  }
 }
